@@ -7,10 +7,29 @@ from ._delete import make_postgres_delete_statement
 from ._update import make_postgres_update_multiple_column_statement
 from ._update import check_parameters_multicol
 import logging
+import signal
 
 db_dict = {}
 
 
+class timeout:
+    """Need because of sometimes query will take very long
+    (cause of connection already killed and tring to execute query)"""
+    def __init__(self, seconds=1, error_message='Timeout'):
+        self.seconds = seconds
+        self.error_message = error_message
+
+    def handle_timeout(self, signum, frame):
+        raise TimeoutError(self.error_message)
+
+    def __enter__(self):
+        signal.signal(signal.SIGALRM, self.handle_timeout)
+        signal.alarm(self.seconds)
+
+    def __exit__(self, type, value, traceback):
+        signal.alarm(0)
+
+
 def get_db(server="default"):
     db_obj = db_dict.get(server, None)
     return db_obj
@@ -18,22 +37,49 @@ def get_db(server="default"):
 print_debug_log = True
 
 
-def pg_server(db_name, username, password, host_address, debug=True, server="default", send_keep_alive_probes=False, socket_idle_time=120):
+def server_connection_check(func):
+    def wrapper(*args, **kwargs):
+        server = kwargs.get('server', 'default')
+        db_obj = get_db(server)
+        if db_obj is None:
+            logging.info('could not got db object for %s' % server)
+            logging.info('db dict %s' % str(db_dict))
+        if db_obj.connection.closed != 0:
+            logging.info('reconnection to db because of connection closed %s' % db_obj.connection.closed)
+            db_obj._make_connection()
+        else:
+            cursor = db_obj.get_cursor()
+            try:
+                with timeout(seconds=10):
+                    cursor.execute('SELECT 1', [])
+            except TimeoutError as e:
+                logging.info('timeout occurred, reconnecting dbs')
+                db_obj._make_connection()
+            except Exception as e:
+                logging.info('reconnection to db because of %s' % e)
+                db_obj._make_connection()
+        return func(*args, **kwargs)
+    return wrapper
+
+
+def pg_server(db_name, username, password, host_address, debug=True, server="default", send_keep_alive_probes=False, socket_idle_time=120, application_name='pg_python'):
     global print_debug_log
+    # no need to make socket so, send_keep_alive_probes will no longer usefull
     params_map = {
         'dbname'  : db_name,
         'user'    : username,
         'password': password,
         'host'    : host_address,
-        'send_keep_alive_probes': send_keep_alive_probes,
-        'socket_idle_time': socket_idle_time,
+        'application_name': application_name
     }
     db_obj = Db(params_map)
     db_dict[server] = db_obj
     print_debug_log = debug
+    logging.info('connected to %s with server %s' % (host_address, server))
     return db_obj
 
 
+@server_connection_check
 def write(table, kv_map, server="default"):
     """
     :param table: String.
@@ -52,7 +98,7 @@ def write(table, kv_map, server="default"):
         connection.commit()
 
     except Exception as e:
-        logging.info("Db Cursor Write Error: %s" % e)
+        logging.error("Db Cursor Write Error: %s" % e)
         db_obj_reconnect = Db(db_obj.params)
         db_dict.pop(server)
         db_dict[server] = db_obj_reconnect
@@ -60,6 +106,7 @@ def write(table, kv_map, server="default"):
     return True
 
 
+@server_connection_check
 def read(table, keys_to_get, kv_map, limit=None, order_by=None, order_type=None,
          clause="=", group_by=None, join_clause=' AND ', server="default",
          cols_keep_raw_type=[]):
@@ -98,11 +145,14 @@ def read(table, keys_to_get, kv_map, limit=None, order_by=None, order_type=None,
         return []
 
 
+@server_connection_check
 def update(table, update_kv_map, where_kv_map, clause='=', server="default"):
     """
     :param table: table name, type string
     :param update_kv_map: the NEW keyvalue map for values to be updated
     :param where_kv_map: the kv map to search for values, all values ARE ANDed.
+    :param clause:
+    :param server:
     :return: Success or Failure.
     """
     global print_debug_log
@@ -113,18 +163,19 @@ def update(table, update_kv_map, where_kv_map, clause='=', server="default"):
                                                      clause, print_debug_log)
     try:
         cursor.execute(command, values)
-        return_dict = {'Status': True}
-        logging.info("%s Record(s) Updated", cursor.rowcount)
+        if print_debug_log:
+            logging.info("%s Record(s) Updated", cursor.rowcount)
         connection.commit()
     except Exception as e:
-        logging.warning("Db Cursor Update Error: %s" % e)
+        logging.error("Db Cursor Update Error: %s" % e)
         db_obj_reconnect = Db(db_obj.params)
         db_dict.pop(server)
         db_dict[server] = db_obj_reconnect
         return False
-    return return_dict
+    return True
 
 
+@server_connection_check
 def read_raw(command, values, server="default"):
     """
     :param table: String
@@ -149,6 +200,7 @@ def read_raw(command, values, server="default"):
         return []
 
 
+@server_connection_check
 def write_raw(command, values, server="default"):
     """
     :params command, values. Execution commands dirctly for postgres
@@ -169,6 +221,7 @@ def write_raw(command, values, server="default"):
     return True
 
 
+@server_connection_check
 def update_raw(command, server="default"):
     """
     Update statement in the raw format,
@@ -216,7 +269,7 @@ def print_fail(s):
 
 
 def close(server="default"):
-    logging.info("Closing connection for server %s" %server)
+    logging.warning("Closing connection for server %s" %server)
     db_obj = get_db(server)
     connection = db_obj.get_connection()
     try:
@@ -232,6 +285,7 @@ def close_all():
         close(server)
 
 
+@server_connection_check
 def delete(table, where_kv_map, server="default"):
     """
     Delete the rows resulting from the mentined kv map. No limit.
@@ -246,10 +300,10 @@ def delete(table, where_kv_map, server="default"):
     command, values = make_postgres_delete_statement(table, where_kv_map, print_debug_log)
     try:
         cursor.execute(command, values)
-        logging.info("%s Record(s) Deleted" %cursor.rowcount)
+        logging.warning("%s Record(s) Deleted" %cursor.rowcount)
         connection.commit()
     except Exception as e:
-        logging.warning("Db Cursor Delete Error: %s" % e)
+        logging.error("Db Cursor Delete Error: %s" % e)
         db_obj_reconnect = Db(db_obj.params)
         db_dict.pop(server)
         db_dict[server] = db_obj_reconnect
@@ -283,6 +337,7 @@ def check_parameters(column_to_update, columns_to_query_lst, query_values_dict_l
     return True
 
 
+@server_connection_check
 def update_multiple(table, column_to_update, columns_to_query_lst,
                     query_values_dict_lst, server="default", typecast = ""):
     """
@@ -313,10 +368,9 @@ def update_multiple(table, column_to_update, columns_to_query_lst,
                                                               typecast_suffix=typecast)
     try:
         cursor.execute(command, values)
-        return_dict = {'Status': True}
+        return_dict = {'status': True}
         count = cursor.rowcount
         return_dict['rowcount'] = count
-
         connection.commit()
     except Exception as e:
         logging.warning("Db Cursor update_multiple Error: %s" % e)
@@ -327,8 +381,8 @@ def update_multiple(table, column_to_update, columns_to_query_lst,
     return return_dict
 
 
-def update_multiple_col(table, columns_to_update_lst, columns_to_query_lst, query_values_dict_lst,
-                        server="default"):
+@server_connection_check
+def update_multiple_col(table, columns_to_update_lst, columns_to_query_lst, query_values_dict_lst, server="default"):
     """
     Multiple update support in pg_python
     :param table: table to update into, string
@@ -387,6 +441,7 @@ def check_multiple_insert_param(columns_to_insert, insert_values_dict_lst):
     return True
 
 
+@server_connection_check
 def insert_multiple(table, columns_to_insert_lst, insert_values_dict_lst, server="default"):
     """
     Multiple row insert in pg_python
